/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class Anim;

#ifndef __Anim_h__
#define __Anim_h__

#include "List.h"
#include "Pageable.h"
#include "Sprite.h"
#include "Movable.h"
#include "String.h"
#include "Exception.h"
#include "Tji.h"

/**
   An Anim (animation) is a sprite list (a sequence of drawings) which can be print in a Page (Pageable), can be move and put on or under the Background, can catch a Key (an action on the keyboard).
   It's important to watch all the properties of a Pageable class (especially Walkable which simulates parallelism (interlaces steps of the objects)).
   @author David FAUTHOUX
   @see Page
   @see Background
   @see Key
   @see Keyer
   @see Walkable
*/
class Anim:public Pageable,public Movable {
 protected:
  ///The sequence of the sprite
  List< Sprite * > list;
  ///The vertical, horizontal position of the animation (all the sprites)
  Tji pos;
  ///The gap to unwedge the animation, it will be painted at pos.j+decal.j,pos.i+decal.i
  Tji decal;
  ///The size of the animation (all the sprites)
  Tji size;
  ///The color of the animation (if colored)
  Pixel color;
  ///If colored
  bool colored;
  /**
     If 'on' the Background. Means : painted relatively of the Background coordinates, but do not respects the default Sprite rules (no pixel printed if in a second level zone) : so, printed every time totaly (if in the screen of course).
     @see Coeur
  */
  bool on;
  /**
     If 'to' the Background. Means : painted relatively of the upper left corner of the screen (not relatively to the Background coordinates), and do not respects the default Sprite rules (no pixel printed if in a second level zone) : so, printed every time totaly (if in the screen of course).
     @see CharacterMot
     @see Trou
  */
  bool to;
  /**
     If 'under' the Background. Means : painted in the Background pixel array and not in the screen, directly in the data array !
     @see Boue
  */
  bool under;

  /**
     An anim can be real-time color painted (passing the pixel through a modifiying colors palette)
     @see Bonhomme
   */
  const PixelPalette *pal;

  /**
     Checks if the destructor has to deleted the Sprites (in the list). Needed because of optimization : rather than re-create an animation when a new comes, I prefer copy one already constructed - with the 'Anim(Anim& a)' constructor.
     @see CaseGestion
   */
  const bool deleter;

  /**
     Used when the Anim is also set 'to' ( see setTo() ) : with 'uppest', the Anim will be on all the others.
     @see Trou
   */
  bool uppest;

  /**
     The function to define for pseudo-parallelism.
     @see Walkable
  */
  int virtual virtualWalk() throw();
 public:
  /**
     This constructor load the file. It defines also the 'slow' integer needed for Walkable.
     @see Walkable
     @exception Throws various exceptions (unable to open file,etc.)
  */
  Anim(const String& nameFile,int slow,char listSeparator) throw (Exception);
  /**
     Use this constructor to copy an animation. It puts the 'deleter' boolean to false.
   */
  Anim(Anim& a) throw ();
  ///
  virtual ~Anim() throw ();

  /**
     Paints the current Sprite of the animation according to the animation properties.
   */
  void virtual paint(WindowPipe& windowPipe) throw();
  ///Sets the animation's position to the first Sprite and activate it.
  void virtual reinit() throw();
  ///Moves the animations (all the Sprites of course) to this vertical, horizontal positions.
  void virtual move(int j,int i) throw();
  ///Vertical, horizontal positions of the animation.
  void virtual getPos(int& j,int& i) const throw();

  /**
     The animation can catch the Key if you derive this function (and add it to the Keyer...)
     @see Keyer
  */
  void virtual keyAction(const Key& key) throw() {}

  /**
     Sets 'on' the Background. Means : painted relatively of the Background coordinates, but do not respects the default Sprite rules (no pixel printed if in a second level zone) : so, printed every time totaly (if in the screen of course).
     @see Coeur
  */
  void virtual setOn() throw();

  /**
     Used when the Anim is also set 'to' ( see setTo() ) : with 'uppest', the Anim will be on all the others.
     @see Trou
  */
  void virtual setUppest() throw();

  /**
     Sets 'to' the Background. Means : painted relatively of the upper left corner of the screen (not relatively to the Background coordinates), and do not respects the default Sprite rules (no pixel printed if in a second level zone) : so, printed every time totaly (if in the screen of course).
     @see CharacterMot
  */
  void virtual setTo() throw();
  /**
     Sets 'under' the Background. Means : painted in the Background pixel array and not in the screen, directly in the data array !
     @see Boue
  */
  void virtual setUnder() throw();

  /**
     Put the palette to this reference, so the pixels will be passed through, and colored. Attention : It is different than 'setColor'.
     @see Bonhomme
   */
  void virtual setPalette(const PixelPalette& pal) throw();

  /**
     Coloring an animation does not mean changing the pixels values, but means change the way the animation will be painted. If the animation is 'colored', it will be painted like that : all the black pixels (0 value) will not be painted, all the others will be painted with the color 'col'. Also, will be added a black shadow on the verge of the Sprite.
     @see CharacterMot
   */
  void virtual setColor(Pixel col) throw();
  ///Undoes the 'setColor' function.
  void virtual restoreColor() throw();

  ///The vertical, horizontal size of the animation.
  void getSize(int& j,int& i) const throw();

  ///Revert the Sprite sequence. The final Sprite becomes the first.
  void revert() throw();//inverse la liste
  
};


#endif
